Package: gpbiometrics
Type: Package
Title: Process and Report Gazepoint Biometrics Data
Version: 0.1.0
Authors@R: person(given = "Stefanos", family = "Balaskas", email = "s.balaskas@ac.upatras.gr", role = c("aut", "cre"))
Description: Imports, validates, quality-checks, preprocesses, summarises, synchronises,
        models, plots, and reports Gazepoint Biometrics exports. The package focuses
        on Gazepoint-specific biometric channels such as GSR/EDA, heart rate,
        interbeat intervals, pulse signals, engagement dial, TTL markers, and
        synchronisation fields that can be combined with Gazepoint GP3 and GP3 HD
        eye-tracking workflows.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
VignetteBuilder: knitr
Suggests: ggplot2, knitr, reticulate, rmarkdown, shiny, testthat (>=
        3.0.0)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-28 01:00:37 UTC; Stefanos-PC
Author: Stefanos Balaskas [aut, cre]
Maintainer: Stefanos Balaskas <s.balaskas@ac.upatras.gr>
Repository: CRAN
Date/Publication: 2026-07-04 07:10:15 UTC
