% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-contracts.R
\name{standardise_gazepoint_plot_contract}
\alias{standardise_gazepoint_plot_contract}
\title{Standardise a Gazepoint plot return contract}
\usage{
standardise_gazepoint_plot_contract(
  plot,
  plot_data = NULL,
  settings = list(),
  interpretation_notes = NULL,
  plot_type = NULL
)
}
\arguments{
\item{plot}{A ggplot object.}

\item{plot_data}{Optional data frame used to create the plot.}

\item{settings}{Optional list of plot settings.}

\item{interpretation_notes}{Optional character vector of interpretation notes.}

\item{plot_type}{Optional short plot-type label.}
}
\value{
A ggplot object with standardized attributes.
}
\description{
Adds consistent attributes to a ggplot object so plotting helpers can be
tested and reused in automated reports.
}
