% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mad-artifact-typology.R
\name{flag_gazepoint_mad_artifacts}
\alias{flag_gazepoint_mad_artifacts}
\title{Flag MAD-based EDA wearable artifacts}
\usage{
flag_gazepoint_mad_artifacts(
  dat,
  eda_col = "GSR_US",
  time_col = NULL,
  group_cols = NULL,
  mad_multiplier = 8,
  flatline_tolerance = 1e-06,
  flatline_min_run = 5,
  wall_abs_change = NULL,
  output_prefix = "mad"
)
}
\arguments{
\item{dat}{A data frame.}

\item{eda_col}{Numeric EDA/conductance column.}

\item{time_col}{Optional time column for ordering within group.}

\item{group_cols}{Optional grouping columns.}

\item{mad_multiplier}{MAD multiplier used for robust thresholding.}

\item{flatline_tolerance}{Maximum absolute sample-to-sample change treated
as flatline.}

\item{flatline_min_run}{Minimum consecutive flatline samples.}

\item{wall_abs_change}{Optional absolute change threshold for wall artifacts.}

\item{output_prefix}{Prefix for output columns.}
}
\value{
A data frame with artifact flags and artifact-summary attributes.
}
\description{
Flags dependency-light, subject-specific EDA artifact categories using robust
median absolute deviation (MAD) logic. The categories are heuristic QC
labels: step artifacts, needle artifacts, flatline artifacts, and wall
artifacts.
}
