% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sync.R
\name{join_gazepoint_biometrics_to_master}
\alias{join_gazepoint_biometrics_to_master}
\title{Join Gazepoint Biometrics to a master table}
\usage{
join_gazepoint_biometrics_to_master(master, biometrics, by, all_x = TRUE)
}
\arguments{
\item{master}{A master gaze or analysis table.}

\item{biometrics}{A Gazepoint Biometrics data frame.}

\item{by}{Character vector of key columns used for joining.}

\item{all_x}{Logical. Should all rows from \code{master} be retained?}
}
\value{
A data frame with biometric columns joined to the master table.
}
\description{
Convenience wrapper around \code{sync_gazepoint_biometrics_with_gaze()} for joining
biometric data to a \code{gp3tools}-style master table or any other analysis-ready
gaze table.
}
