% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{check_gazepoint_biometric_columns}
\alias{check_gazepoint_biometric_columns}
\title{Check Gazepoint biometric columns}
\usage{
check_gazepoint_biometric_columns(data)
}
\arguments{
\item{data}{A data frame imported from a Gazepoint export.}
}
\value{
A data frame describing expected columns, their signal family,
interpretation, and whether they are present.
}
\description{
Checks whether a data frame contains known Gazepoint Biometrics columns.
This function does not assume that the channels are active. It only checks
whether the expected columns are present.
}
