% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/changepoint-scr-recovery.R
\name{extract_gazepoint_scr_recovery_times}
\alias{extract_gazepoint_scr_recovery_times}
\title{Extract SCR recovery times}
\usage{
extract_gazepoint_scr_recovery_times(
  dat,
  eda_col = "GSR_US",
  time_col = "CNT",
  event_onset_col = NULL,
  group_cols = NULL,
  pre_onset_baseline_s = 2,
  peak_window_s = 5,
  recovery_window_s = 20
)
}
\arguments{
\item{dat}{A data frame.}

\item{eda_col}{Numeric EDA/conductance column.}

\item{time_col}{Numeric time column.}

\item{event_onset_col}{Optional event onset column. Finite values are treated
as event onsets.}

\item{group_cols}{Optional grouping columns.}

\item{pre_onset_baseline_s}{Baseline window before event onset.}

\item{peak_window_s}{Window after onset used to find the response peak.}

\item{recovery_window_s}{Window after peak used to find recovery.}
}
\value{
A list with \code{overview}, \code{recovery_table}, and \code{settings}.
}
\description{
Extracts 50 percent half-recovery time (\code{rec.t2}) and 63 percent recovery
time (\code{rec.tc}) for skin conductance responses from an EDA waveform and
event onsets.
}
