% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdr-rsa.R
\name{extract_gazepoint_pdr_signals}
\alias{extract_gazepoint_pdr_signals}
\title{Extract PPG-derived respiration proxy signals}
\usage{
extract_gazepoint_pdr_signals(
  dat,
  ppg_col = "HRP",
  time_col = "CNT",
  group_cols = NULL,
  sampling_rate = NULL,
  min_peak_distance_s = 0.3,
  smooth_window = 5,
  respiration_band = c(0.1, 0.6),
  pdr_resample_rate = 4
)
}
\arguments{
\item{dat}{A data frame containing a PPG/pulse waveform.}

\item{ppg_col}{Numeric PPG/pulse waveform column, often \code{HRP}.}

\item{time_col}{Numeric time column.}

\item{group_cols}{Optional grouping columns.}

\item{sampling_rate}{Optional sampling rate in Hz. If \code{NULL}, estimated from
\code{time_col}.}

\item{min_peak_distance_s}{Minimum plausible distance between pulse peaks.}

\item{smooth_window}{Number of samples used for simple moving-average
smoothing before peak detection.}

\item{respiration_band}{Expected respiration-frequency band in Hz.}

\item{pdr_resample_rate}{Resampling rate used for spectral estimation of PDR
proxy signals.}
}
\value{
A list with \code{overview}, \code{pulse_features}, \code{pdr_timeseries},
\code{pdr_summary}, and \code{settings}.
}
\description{
Extracts dependency-light PPG-derived respiration proxy features from a
Gazepoint pulse/PPG waveform. The function estimates respiration-modulated
pulse features such as respiratory-induced intensity variability (RIIV),
pulse amplitude variability (PAV), pulse width variability (PWV), and
pulse-rate variability (PRV).
}
\details{
These are proxy respiratory features. They should not be treated as a
replacement for a respiration belt or clinical respiratory measurement.
}
