% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctsi-bridge.R
\name{prepare_gazepoint_ctsi_input}
\alias{prepare_gazepoint_ctsi_input}
\title{Prepare Gazepoint EDA data for CTSI sparse deconvolution workflows}
\usage{
prepare_gazepoint_ctsi_input(
  dat,
  eda_col = "GSR_US",
  time_col = "CNT",
  group_cols = NULL,
  event_onset_col = NULL,
  event_name_col = NULL,
  sampling_rate = NULL,
  tau0_range = c(2, 4),
  tau1_range = c(0.5, 1),
  sparsity_grid = c(0.001, 0.01, 0.1, 1),
  output_dir = NULL,
  prefix = "gazepoint_ctsi"
)
}
\arguments{
\item{dat}{A data frame containing EDA data.}

\item{eda_col}{Numeric EDA/conductance column.}

\item{time_col}{Numeric time column.}

\item{group_cols}{Optional grouping columns, such as participant/session.}

\item{event_onset_col}{Optional event onset column.}

\item{event_name_col}{Optional event/condition column.}

\item{sampling_rate}{Optional sampling rate in Hz.}

\item{tau0_range}{Candidate slow time-constant range.}

\item{tau1_range}{Candidate fast time-constant range.}

\item{sparsity_grid}{Candidate sparsity penalties.}

\item{output_dir}{Optional directory for CSV export.}

\item{prefix}{Output file prefix.}
}
\value{
A list with \code{overview}, \code{signal_table}, \code{event_table},
\code{ctsi_config}, \code{ctsi_notes}, \code{written_files}, and \code{settings}.
}
\description{
Prepares signal, event, and configuration tables for downstream
continuous-time system identification (CTSI) sparse EDA deconvolution
workflows. This function does not implement the full Amin-Faghih CTSI solver
in R. It creates reproducible input objects and optional CSV files for
external CTSI implementations.
}
