% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature-inventory-format.R
\name{summarise_gazepoint_biometrics_feature_inventory}
\alias{summarise_gazepoint_biometrics_feature_inventory}
\title{Summarise the formatted gpbiometrics feature inventory}
\usage{
summarise_gazepoint_biometrics_feature_inventory(formatted_inventory = NULL)
}
\arguments{
\item{formatted_inventory}{Optional table returned by
\code{format_gazepoint_biometrics_feature_inventory()}.}
}
\value{
A list with domain, method-family, and user-level summaries.
}
\description{
Summarise the formatted gpbiometrics feature inventory
}
