% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny-and-preregistration.R
\name{run_gpbiometrics_shiny}
\alias{run_gpbiometrics_shiny}
\title{Launch a lightweight gpbiometrics Shiny dashboard}
\usage{
run_gpbiometrics_shiny()
}
\value{
A Shiny application object, launched for interactive use.
}
\description{
Launches an optional local Shiny interface for inspecting a Gazepoint
Biometrics CSV file. Shiny is optional and is not required to use the package.
}
