% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pupil-and-saccade-diagnostics.R
\name{plot_gazepoint_saccade_main_sequence}
\alias{plot_gazepoint_saccade_main_sequence}
\title{Plot Gazepoint saccade main-sequence diagnostics}
\usage{
plot_gazepoint_saccade_main_sequence(
  dat,
  amplitude_col = NULL,
  peak_velocity_col = NULL,
  group_col = NULL,
  log_axes = TRUE,
  add_smoother = TRUE,
  main = "Gazepoint saccade main-sequence diagnostic"
)
}
\arguments{
\item{dat}{A saccade-level data frame.}

\item{amplitude_col}{Saccade amplitude column.}

\item{peak_velocity_col}{Peak velocity column.}

\item{group_col}{Optional grouping column.}

\item{log_axes}{Logical. If \code{TRUE}, use log10 axes.}

\item{add_smoother}{Logical. If \code{TRUE}, add a lowess curve.}

\item{main}{Plot title.}
}
\value{
Invisibly returns the plotted data and settings.
}
\description{
Plots saccade amplitude against peak velocity. The function expects
saccade-level amplitude and peak-velocity columns. If raw sample-level data
are supplied, users should first derive saccade-level kinematics with a
validated fixation/saccade detector.
}
