% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/workflow.R
\name{summarise_gazepoint_biometrics_workflow}
\alias{summarise_gazepoint_biometrics_workflow}
\title{Summarise a Gazepoint Biometrics workflow object}
\usage{
summarise_gazepoint_biometrics_workflow(workflow)
}
\arguments{
\item{workflow}{A workflow object returned by
\code{run_gazepoint_biometrics_workflow()}.}
}
\value{
A one-row data frame summarising the workflow.
}
\description{
Creates a compact summary table from an object returned by
\code{run_gazepoint_biometrics_workflow()}.
}
