% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aoi-biometrics.R
\name{plot_gazepoint_aoi_biometrics}
\alias{plot_gazepoint_aoi_biometrics}
\title{Plot AOI-linked biometric summaries}
\usage{
plot_gazepoint_aoi_biometrics(
  x,
  value_col = "mean_value",
  aoi_col = "aoi_label",
  signal_col = "signal",
  group_col = NULL,
  plot_type = c("boxplot", "point", "line"),
  title = NULL
)
}
\arguments{
\item{x}{A \code{gazepoint_aoi_biometrics_summary},
\code{gazepoint_aoi_biometrics_model_data}, or data frame.}

\item{value_col}{Value column to plot.}

\item{aoi_col}{AOI label column.}

\item{signal_col}{Signal label column.}

\item{group_col}{Optional grouping column.}

\item{plot_type}{\code{"boxplot"}, \code{"point"}, or \code{"line"}.}

\item{title}{Optional plot title.}
}
\value{
A ggplot object with plot data stored in attributes.
}
\description{
Plots AOI-biometric summary values as a ggplot object.
}
