% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/folder.R
\name{import_gazepoint_biometric_folder}
\alias{import_gazepoint_biometric_folder}
\title{Import a folder of Gazepoint Biometrics exports}
\usage{
import_gazepoint_biometric_folder(
  path,
  pattern = "\\\\.csv$",
  recursive = FALSE,
  include_fixations = TRUE,
  include_all_gaze = TRUE,
  include_other_csv = FALSE,
  na = c("", "NA", "NaN")
)
}
\arguments{
\item{path}{Folder containing Gazepoint CSV exports.}

\item{pattern}{Regular expression used to identify candidate CSV files.}

\item{recursive}{Should subfolders be searched?}

\item{include_fixations}{Should files with \code{"fixation"} in the file name be
included?}

\item{include_all_gaze}{Should files with \code{"all_gaze"} in the file name be
included?}

\item{include_other_csv}{Should other CSV files be attempted? The default is
\code{FALSE} to avoid accidentally trying to parse multi-section
\code{Data_Summary_export} files as rectangular data.}

\item{na}{Values that should be treated as missing.}
}
\value{
A data frame with all imported rows combined. The output includes a
\code{source_file} column and has class \code{"gazepoint_biometrics_folder"}.
}
\description{
Reads rectangular Gazepoint CSV exports from a folder and combines files that
contain at least one known Gazepoint Biometrics column. This function is
designed for all-gaze and fixation-style exports. Multi-section Gazepoint
Data Summary files should be parsed separately.
}
